"
    Copyright (C) 2015-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.
"
"{ Package: 'jv:llvm_s/tests' }"

"{ NameSpace: Smalltalk }"

TestCase subclass:#LLVMTypeTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'LLVM-S-Core-Types-Tests'
!

!LLVMTypeTests class methodsFor:'documentation'!

copyright
"
    Copyright (C) 2015-now Jan Vrany

    This code is not an open-source (yet). You may use this code
    for your own experiments and projects, given that:

    * all modification to the code will be sent to the
      original author for inclusion in future releases
    * this is not used in any commercial software

    This license is provisional and may (will) change in
    a future.
"
! !

!LLVMTypeTests methodsFor:'tests - function types'!

test_function_01
    | ftype |

    ftype := LLVMType function: { } returning: LLVMType void.

    self assert: ftype isFunctionType.
    self assert: ftype returnType isVoidType.
    self assert: ftype numParams == 0.
    self assert: ftype parameterTypes = #().

    ftype := LLVMType function: { LLVMType intptr } returning: LLVMType void.

    self assert: ftype isFunctionType.
    self assert: ftype returnType isVoidType.
    self assert: ftype numParams == 1.
    self assert: ftype parameterTypes = { LLVMType intptr }.

    ftype := LLVMType function: { LLVMType intptr . LLVMType intptr pointer } returning: LLVMType half.

    self assert: ftype isFunctionType.
    self assert: ftype returnType isHalfType.
    self assert: ftype numParams == 2.
    self assert: ftype parameterTypes = { LLVMType intptr . LLVMType intptr pointer }

    "Created: / 13-08-2015 / 18:38:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!LLVMTypeTests methodsFor:'tests - other'!

test_dumpString_01

    self assert: LLVMType int1 dumpString = 'i1'.

    "Created: / 02-09-2015 / 21:43:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_sizeInBits_01

    self assert: LLVMType int1 sizeInBits == 1.
    self assert: LLVMType int8 sizeInBits == 8.
    self assert: LLVMType int16 sizeInBits == 16.
    self assert: LLVMType int32 sizeInBits == 32.
    self assert: LLVMType int64 sizeInBits == 64.

    self assert: LLVMType half sizeInBits == 16.
    self assert: LLVMType float sizeInBits == 32.
    self assert: LLVMType double sizeInBits == 64.
    self assert: LLVMType x86fp80 sizeInBits == 80.
    self assert: LLVMType fp128 sizeInBits == 128.

    "Created: / 13-08-2015 / 18:46:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!LLVMTypeTests methodsFor:'tests - sequential types'!

test_array_01
    | type ptype |

    type := LLVMType int8.
    ptype := type array:13.

    self assert: ptype isArrayType.
    self assert: ptype elementType = type.
    self assert: ptype sizeInElements = 13.

    type := LLVMType x86mmx.
    ptype := type array: 13.

    self assert: ptype isArrayType.
    self assert: ptype elementType = type.
    self assert: ptype sizeInElements = 13.

    "Created: / 13-08-2015 / 19:08:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_pointer_01
    | type ptype |

    type := LLVMType int8.
    ptype := type pointer.

    self assert: ptype isPointerType.
    self assert: ptype elementType = type.

    type := LLVMType x86mmx.
    ptype := type pointer.

    self assert: ptype isPointerType.
    self assert: ptype elementType = type

    "Created: / 13-08-2015 / 19:03:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_vector_01
    | type ptype |

    type := LLVMType int8.
    ptype := type vector:13.

    self assert: ptype isVectorType.
    self assert: ptype elementType = type.
    self assert: ptype sizeInElements = 13.

    type := LLVMType x86mmx.
    ptype := type vector: 13.

    self assert: ptype isVectorType.
    self assert: ptype elementType = type.
    self assert: ptype sizeInElements = 13.

    "Created: / 13-08-2015 / 19:09:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!LLVMTypeTests methodsFor:'tests - struct'!

test_struct_01
    | struct |

    struct := LLVMType struct: { LLVMType int8 . LLVMType char pointer }.

    self assert: struct isStructType.
    self deny: struct isPacked.
    self deny: struct isPacked.
    self assert: struct numElements == 2.
    self assert: struct elementTypes = { LLVMType int8 . LLVMType char pointer }.
    self assert: (struct elementTypeAt: 1) = LLVMType int8.
    self assert: (struct elementTypeAt: 2) = LLVMType char pointer .

    self should: [ struct elementTypeAt: 0 ] raise: LLVMTypeError.
    self should: [ struct elementTypeAt: 3 ] raise: LLVMTypeError.

    self assert: struct pointer elementType = struct

    "Created: / 14-08-2015 / 06:16:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!LLVMTypeTests class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

